// dumbsubmenus.cpp
// by Glenn G. Chappell
// September 2003
//
// For CS 381
// Demo of GLUT submenus

#include <iostream>
using std::cerr;
using std::endl;
#include <string>
using std::string;
#include <sstream>
using std::ostringstream;
#include <stdlib.h>  // Some versions of MS-Vis C++ have broken <cstdlib>
//using std::exit;
#include <GL/glut.h> // GLUT stuff, includes OpenGL headers as well


// Global variables
// Window
const int startwinsize = 400;       // Starting window width & height, in pixels

// Keyboard
const int ESCKEY = 27;              // ASCII value of escape character

// Menus
int leftmouse_menu,                 // GLUT menu ID's
    rightmouse_menu,
    compliments_menu,
    insults_menu,
    other_menu,
    letter_menu,
    number_menu;


// display
// The GLUT display function
void display()
{
   glClear(GL_COLOR_BUFFER_BIT);

   glColor3d(0.9, 0.5, 0.1);
   glBegin(GL_TRIANGLES);  // Draw a triangle
      glVertex2d(-0.7, -0.7);
      glVertex2d( 0.7, -0.7);
      glVertex2d( 0.0,  0.7);
   glEnd();

   glutSwapBuffers();
}


// idle
// The GLUT idle function
void idle()
{
   // Print OpenGL errors, if there are any (for debugging)
   if (GLenum err = glGetError())
   {
      cerr << "OpenGL ERROR: " << gluErrorString(err) << endl;
   }
}


// reshape
// The GLUT reshape function
void reshape(int w, int h)
{
   glViewport(0, 0, w, h);

   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   // We set up coordinate system so that aspect ratios are always correct,
   //  and the region from -1..1 in x & y always just fits in the viewport.
   if (w > h)
   {
      gluOrtho2D(-double(w)/h, double(w)/h, -1., 1.);
   }
   else
   {
      gluOrtho2D( -1., 1., -double(h)/w, double(h)/w);
   }
}


// keyboard
// The GLUT keyboard function
void keyboard(unsigned char key, int x, int y)
{
   switch (key)
   {
   case ESCKEY:  // ESC: Quit
      exit(0);
      break;
   default:      // Any other ASCII key refreshes display
      glutPostRedisplay();
      break;
   }
}


// handle_leftmouse_menu
// Menu handling callback function for leftmouse menu
void handle_leftmouse_menu(int value)
{
   switch (value)
   {
   case 999:  // Menu items do nothing
      break;
   }
}


// handle_letter_menu
// Menu handling callback function for letter menu
void handle_letter_menu(int value)
{
   switch (value)
   {
   case 999:  // Menu items do nothing
      break;
   }
}


// handle_number_menu
// Menu handling callback function for number menu
void handle_number_menu(int value)
{
   switch (value)
   {
   case 999:  // Menu items do nothing
      break;
   }
}


// handle_rightmouse_menu
// Menu handling callback function for rightmouse menu
void handle_rightmouse_menu(int value)
{
   switch (value)
   {
   case 999:  // Menu items do nothing
      break;
   }
}


// handle_compilments_menu
// Menu handling callback function for compliments menu
void handle_compliments_menu(int value)
{
   switch (value)
   {
   case 999:  // Menu items do nothing
      break;
   }
}


// handle_insults_menu
// Menu handling callback function for insults menu
void handle_insults_menu(int value)
{
   switch (value)
   {
   case 999:  // Menu items do nothing
      break;
   }
}


// handle_other_menu
// Menu handling callback function for other menu
void handle_other_menu(int value)
{
   switch (value)
   {
   case 999:  // Menu items do nothing
      break;
   }
}


// make_menus
// Creates all menus
void make_menus()
{
   compliments_menu = glutCreateMenu(handle_compliments_menu);
   glutAddMenuEntry("You look great!", 999);
   glutAddMenuEntry("I have always admired you.", 999);
   glutAddMenuEntry("Good job!", 999);

   insults_menu = glutCreateMenu(handle_insults_menu);
   glutAddMenuEntry("You are an idiot.", 999);
   glutAddMenuEntry("You stink!", 999);
   glutAddMenuEntry("Don't you know ANYTHING?!?!", 999);

   other_menu = glutCreateMenu(handle_other_menu);
   glutAddMenuEntry("Hi.", 999);
   glutAddMenuEntry("Do you have the time?", 999);
   glutAddMenuEntry("Excuse me.", 999);
   glutAddMenuEntry("Would you like fries with that?", 999);

   rightmouse_menu = glutCreateMenu(handle_rightmouse_menu);
   glutAddMenuEntry("No menu items do anything", 999);
   glutAddMenuEntry("Escape to quit", 999);
   glutAddMenuEntry("------------------------", 999);
   glutAddSubMenu("Compliments", compliments_menu);
   glutAddSubMenu("Insults", insults_menu);
   glutAddSubMenu("Other comments", other_menu);
   glutAttachMenu(GLUT_RIGHT_BUTTON);

   letter_menu = glutCreateMenu(handle_letter_menu);
   for (char ch='A'; ch<='Z'; ++ch)
   {
      glutAddMenuEntry(string(1, ch).c_str(), 999);
   }

   number_menu = glutCreateMenu(handle_number_menu);
   for (int n=1; n<= 10; ++n)
   {
      ostringstream os;
      os << n;
      glutAddMenuEntry(os.str().c_str(), 999);
   }

   leftmouse_menu = glutCreateMenu(handle_leftmouse_menu);
   glutAddMenuEntry("No menu items do anything", 999);
   glutAddMenuEntry("Escape to quit", 999);
   glutAddMenuEntry("------------------------", 999);
   glutAddSubMenu("Pick a letter", letter_menu);
   glutAddSubMenu("Pick a number", number_menu);
   glutAttachMenu(GLUT_LEFT_BUTTON);
}


// init
// General initialization
// Called by main.
void init() 
{
   make_menus();

   glClearColor(1., 1., 1., 0.);
}


int main(int argc, char ** argv)
{
   // Initialize OpenGL/GLUT
   glutInit(&argc, argv);
   glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB);

   // Make a window
   glutInitWindowSize(startwinsize, startwinsize);
   glutInitWindowPosition(50, 50);
   glutCreateWindow("CS 381 - Submenu Demo - Right or left mouse for menu");

   // Initialize & register callbacks
   init();
   glutDisplayFunc(display);
   glutKeyboardFunc(keyboard);
   glutIdleFunc(idle);
   glutReshapeFunc(reshape);

   // Do something
   glutMainLoop();

   return 0;
}
